Package: clusterGeneration
Version: 1.3.8
Date: 2023-08-15
Title: Random Cluster Generation (with Specified Degree of Separation)
Author: Weiliang Qiu <weiliang.qiu@gmail.com>, Harry Joe
        <harry@stat.ubc.ca>.
Maintainer: Weiliang Qiu <weiliang.qiu@gmail.com>
Depends: R (>= 3.5.0), MASS
Description: We developed the clusterGeneration package to provide functions 
        for generating random clusters, generating random 
        covariance/correlation matrices,
        calculating a separation index (data and population version)
        for pairs of clusters or cluster distributions, and 1-D and 2-D
        projection plots to visualize clusters.  The package also
        contains a function to generate random clusters based on
        factorial designs with factors such as degree of separation,
        number of clusters, number of variables, number of noisy
        variables.
License: GPL (>= 2)
Packaged: 2023-08-16 02:01:35 UTC; weiliangqiu
Repository: CRAN
Date/Publication: 2023-08-16 04:20:02 UTC
NeedsCompilation: no
Built: R 4.6.0; ; 2026-04-26 09:54:39 UTC; unix
