
export(genRandomClust, 
       simClustDesign,
       rcorrmatrix,
       genPositiveDefMat,
       sepIndexTheory,
       sepIndexData,
       genOrthogonal,
       getSepProjTheory,
       getSepProjData,
       nearestNeighborSepVal,
       plot1DProjection,
       plot2DProjection,
       viewClusters
      ) 

import(MASS)

    importFrom("graphics", "par", "points", "title")
    importFrom("stats", "cov", "density", "dist", "median", "qbeta",
               "qnorm", "quantile", "rbeta", "rnorm", "runif", "sd",
               "uniroot")
    importFrom("utils", "write.table")
